@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008 Canonical Ltd
+  Copyright (C) 2008 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as 
@@ -21,6 +21,9 @@
 #include <config.h>
 #endif
 
+#include <gconf/gconf.h>
+#include <gconf/gconf-client.h>
+
 #include "launcher-sidebar.h"
 
 #include <gio/gio.h>
@@ -53,7 +56,7 @@ G_DEFINE_TYPE_WITH_CODE (LauncherSidebar,
   LAUNCHER_TYPE_SIDEBAR, \
   LauncherSidebarPrivate))
 
-#define NAUTILUS  "nautilus --no-desktop --browser %s"
+#define NAUTILUS  "xdg-open \"%s\""
 
 struct _LauncherSidebarPrivate
 {
@@ -61,8 +64,7 @@ struct _LauncherSidebarPrivate
   GFile          *bookfile;
   GFileMonitor   *bookfile_mon;
 
-  ClutterActor *bg;
-  
+  ClutterActor   *bg;
   ClutterActor   *clip;
   ClutterActor   *grid;
   ClutterActor   *scroll;
@@ -72,6 +74,8 @@ struct _LauncherSidebarPrivate
   GList *links;
   GList *places;
 
+  GSList *excluded_volumes;
+
   ClutterActor *trash;
   ClutterActor *focused;
 
@@ -79,6 +83,29 @@ struct _LauncherSidebarPrivate
   ClutterEffectTemplate *temp;
 };
 
+enum
+{
+  SHORTCUTS_CHANGED,
+  
+  LAST_SIGNAL
+};
+static guint32 _sidebar_signals[LAST_SIGNAL] = { 0 };
+
+GList *
+launcher_sidebar_get_places (LauncherSidebar *sidebar)
+{
+  LauncherSidebarPrivate *priv;
+  GList *places = NULL;
+
+  g_return_val_if_fail (LAUNCHER_IS_SIDEBAR (sidebar), NULL);
+  priv = sidebar->priv;
+
+  places = g_list_concat (places, g_list_copy (priv->shortcuts));
+  places = g_list_concat (places, g_list_copy (priv->places));
+
+  return places;
+}
+
 gboolean
 launcher_sidebar_scroll_event (ClutterActor       *actor,
                               ClutterScrollEvent *event,
@@ -137,6 +164,9 @@ ensure_layout (LauncherSidebar *bar)
   for (s = priv->shortcuts; s; s = s->next)
   {
     ClutterActor *actor = s->data;
+
+    if (!CLUTTER_IS_ACTOR (actor))
+      continue;
     
     clutter_actor_set_position (actor, 0, offset);
 
@@ -148,6 +178,9 @@ ensure_layout (LauncherSidebar *bar)
   {
     ClutterActor *actor = s->data;
 
+    if (!CLUTTER_IS_ACTOR (actor))
+      continue;
+
     clutter_actor_set_position (actor, 0, offset);
 
     offset += bar_height;
@@ -196,12 +229,12 @@ ensure_layout (LauncherSidebar *bar)
     tidy_frame_set_texture (TIDY_FRAME (scroll), frame);
 
     clutter_container_add_actor (CLUTTER_CONTAINER (priv->clip), scroll);
-    clutter_actor_set_size (scroll, bar_height * (N_CATS-2), 5);
+    clutter_actor_set_size (scroll, bar_height * (N_CATS-2) -2, 5);
     clutter_actor_set_anchor_point_from_gravity (scroll, 
                                                  CLUTTER_GRAVITY_CENTER);
     clutter_actor_set_position (scroll, 
-                                bar_width- 10/2+1,
-                                ((bar_height*(N_CATS-2))/2)+2);
+                                bar_width- 10/2,
+                                ((bar_height*(N_CATS-2))/2) + 2);
     clutter_actor_set_opacity (scroll, 100);
     clutter_actor_set_rotation (scroll, CLUTTER_Z_AXIS, 90, 0, 0, 0);
     clutter_actor_show (scroll);
@@ -210,12 +243,79 @@ ensure_layout (LauncherSidebar *bar)
   {
     clutter_actor_set_opacity (priv->scroll, 0);
   }
+
+  g_signal_emit (bar, _sidebar_signals[SHORTCUTS_CHANGED], 0);
 }
 
 
 /*
  * PLACES 
  */
+static void
+on_places_mount (GFile *file, GAsyncResult *res, LauncherShortcut *shortcut)
+{
+  GdkPixbuf *temp;
+  GError *error = NULL;
+  gboolean success;
+  
+  success = g_file_mount_enclosing_volume_finish (file, res, &error);
+
+  if (success)
+  {
+    g_app_info_launch_default_for_uri (g_file_get_uri (file), NULL, &error);
+  }
+
+  temp = launcher_shortcut_get_icon (LAUNCHER_SHORTCUT (shortcut));
+  launcher_notify_popup (launcher_notify_get_default (),
+                  launcher_shortcut_get_label (LAUNCHER_SHORTCUT (shortcut)),
+                         temp, 
+                         -1);
+
+  g_object_unref (temp);
+  g_object_unref (file);
+}
+
+static void
+on_place_clicked (ClutterActor *shortcut, gpointer null)
+{
+  GVfs *vfs;
+  GFile *file;
+  const gchar *uri;
+  GdkPixbuf *temp;
+  GError *error = NULL;
+
+  launcher_shortcut_set_active (LAUNCHER_SHORTCUT (shortcut), FALSE);
+
+  uri = clutter_actor_get_name (shortcut);
+
+  if (!uri)
+    return;
+
+  g_app_info_launch_default_for_uri (uri, NULL, &error);
+
+  if (error)
+  {
+    g_warning ("%s\n", error->message);
+    g_error_free (error);
+
+    vfs = g_vfs_get_default ();
+    file = g_vfs_get_file_for_uri (vfs, uri);
+    g_file_mount_enclosing_volume (file, 0, NULL, NULL, 
+                                   (GAsyncReadyCallback)on_places_mount,
+                                   shortcut);
+
+    return;
+  }
+
+  temp = launcher_shortcut_get_icon (LAUNCHER_SHORTCUT (shortcut));
+  launcher_notify_popup (launcher_notify_get_default (),
+                  launcher_shortcut_get_label (LAUNCHER_SHORTCUT (shortcut)),
+                         temp, 
+                         -1);
+
+  g_object_unref (temp);
+}
+
 static ClutterActor * 
 add_place (LauncherSidebar *sidebar, 
            const gchar *icon_name, 
@@ -224,17 +324,67 @@ add_place (LauncherSidebar *sidebar,
 {
   LauncherSidebarPrivate *priv = sidebar->priv;
   ClutterActor *shortcut;
-   
+  
   shortcut = launcher_shortcut_new (icon_name, name, FALSE);
   clutter_container_add_actor (CLUTTER_CONTAINER (priv->grid), shortcut);
   clutter_actor_set_name (shortcut, command);
-  clutter_actor_show (shortcut);
+  clutter_actor_show_all(shortcut);
+
   priv->places = g_list_append (priv->places, shortcut);
-  g_signal_connect (shortcut, "clicked", G_CALLBACK (on_link_clicked), NULL);
+  g_signal_connect (shortcut, "clicked", G_CALLBACK (on_place_clicked), NULL);
   
   return shortcut;
 }
 
+static gchar *
+get_icon_name_for_place (const gchar *uri)
+{
+  GFile *file;
+  GFileInfo *info;
+  GIcon *icon = NULL;
+  const gchar * const *names;
+  GtkIconTheme *icon_theme;
+  int i;
+
+  file = g_file_new_for_uri (uri);
+  info = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_ICON, 
+                            0, NULL, NULL);
+  if (info)
+    icon = (GIcon*)g_file_info_get_attribute_object (info, 
+                                             G_FILE_ATTRIBUTE_STANDARD_ICON);
+
+  if (!G_IS_THEMED_ICON (icon))
+  {
+    g_object_unref (file);
+    if (info)
+      g_object_unref (info);
+
+    g_print ("Not themed icon: %s\n", uri);
+
+    return g_strdup ("folder");
+  }
+
+  names = g_themed_icon_get_names (G_THEMED_ICON (icon));
+  icon_theme = gtk_icon_theme_get_default ();
+
+  for (i = 0; names[i] != NULL; i++)
+  {
+    if (gtk_icon_theme_has_icon (icon_theme, names[i]))
+    {
+      gchar *ret = g_strdup (names[i]);
+      
+      g_object_unref (file);
+      g_object_unref (info);
+      
+      return ret;
+    }
+  }
+
+  g_object_unref (file);
+  g_object_unref (info);
+
+  return NULL;
+}
 static void
 load_places (LauncherSidebar *bar)
 {
@@ -271,7 +421,7 @@ load_places (LauncherSidebar *bar)
 
     for (i = 0; bookmarks[i]; i++)
     {
-      const gchar *icon_name = "folder";
+      gchar *icon_name;
       gchar *label;
 
       label = strchr (bookmarks[i], ' ');
@@ -290,14 +440,13 @@ load_places (LauncherSidebar *bar)
            && !g_ascii_strcasecmp (bookmarks[i], "network:///"))
       {
         label = _("Network");
-        icon_name = "gnome-fs-network";
       }
       
       if (label)
       {
-        path = g_strdup_printf (NAUTILUS, bookmarks[i]);
-        add_place (bar, icon_name, label, path);
-        g_free (path);
+        icon_name = get_icon_name_for_place (bookmarks[i]);
+        add_place (bar, icon_name, label, bookmarks[i]);
+        g_free (icon_name);
       }
     }
     g_strfreev (bookmarks);
@@ -311,7 +460,7 @@ static void
 on_places_changed (GFileMonitor      *monitor,
                    GFile             *file,
                    GFile             *old_file,
-                   GFileMonitorEvent *event,
+                   GFileMonitorEvent  event,
                    LauncherSidebar   *bar)
 {
   g_return_if_fail (LAUNCHER_IS_SIDEBAR (bar));
@@ -322,10 +471,41 @@ on_places_changed (GFileMonitor      *monitor,
 /*
  * VOLUMES
  */
+static gboolean
+volume_is_excluded (LauncherSidebar *bar, GVolume *volume)
+{
+  LauncherSidebarPrivate *priv = bar->priv;
+  GSList *v;
+  gchar *uid;
+
+  uid = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_HAL_UDI);
+
+  if (!uid)
+    return FALSE;
+
+  for (v = priv->excluded_volumes; v; v = v->next)
+  {
+    if (v->data && strstr (uid, v->data))
+    {
+      g_free (uid);
+      return TRUE;
+    }
+  }
+
+  g_free (uid);
+  return FALSE;
+}
+
+
 static gboolean
 can_eject (GVolume *volume)
 {
-  if (g_volume_can_eject (volume))
+
+  if (!G_IS_VOLUME (volume))
+  {
+    return FALSE;
+  }
+  else if (g_volume_can_eject (volume))
   {
     return TRUE;
   }
@@ -353,7 +533,8 @@ on_volume_mounted (GVolume          *volume,
   if (!success)
     return;
 
-  launcher_shortcut_set_active (LAUNCHER_SHORTCUT (shortcut), FALSE);
+  if (shortcut)
+    launcher_shortcut_set_active (LAUNCHER_SHORTCUT (shortcut), FALSE);
 
   mount = g_volume_get_mount (volume);
   if (!G_IS_MOUNT (mount))
@@ -362,7 +543,7 @@ on_volume_mounted (GVolume          *volume,
   }
 
   root = g_mount_get_root (mount);
-  path = g_file_get_path (root);
+  path = g_file_get_uri (root);
   command = g_strdup_printf (NAUTILUS, path);
 
   gdk_spawn_command_line_on_screen (gdk_screen_get_default (),
@@ -382,7 +563,6 @@ on_volume_clicked (ClutterActor *shortcut, GVolume *volume)
   
   g_return_if_fail (G_IS_VOLUME (volume));
 
-
   mount = g_volume_get_mount (volume);
   if (!G_IS_MOUNT (mount))
   {
@@ -396,7 +576,7 @@ on_volume_clicked (ClutterActor *shortcut, GVolume *volume)
   }
 
   root = g_mount_get_root (mount);
-  path = g_file_get_path (root);
+  path = g_file_get_uri (root);
   command = g_strdup_printf (NAUTILUS, path);
 
   gdk_spawn_command_line_on_screen (gdk_screen_get_default (),
@@ -414,6 +594,58 @@ on_volume_clicked (ClutterActor *shortcut, GVolume *volume)
   g_free (path);
 }
 
+void  
+launcher_sidebar_launch_volume (LauncherSidebar *sidebar,
+                                GVolume         *volume)
+{
+  GMount *mount;
+  GFile *root;
+  gchar *path, *command;
+    
+  g_return_if_fail (G_IS_VOLUME (volume));
+
+  mount = g_volume_get_mount (volume);
+  if (!G_IS_MOUNT (mount))
+  {
+    if (!g_volume_can_mount (volume))
+      return;
+
+    g_volume_mount (volume, 0, NULL, NULL, 
+                    (GAsyncReadyCallback)on_volume_mounted, NULL);
+
+    return;
+  }
+
+  root = g_mount_get_root (mount);
+  path = g_file_get_uri (root);
+  command = g_strdup_printf (NAUTILUS, path);
+
+  gdk_spawn_command_line_on_screen (gdk_screen_get_default (),
+                                    command, NULL);
+
+  g_free (command);
+  g_free (path);
+}
+
+static void
+on_volume_ejected (GVolume          *volume,
+                   GAsyncResult     *res,
+                   LauncherShortcut *shortcut)
+{
+  g_volume_eject_finish (volume, res, NULL);
+  clutter_actor_set_opacity (CLUTTER_ACTOR (shortcut), 175);
+}
+
+static void
+on_mount_unmounted (GMount           *mount,
+                    GAsyncResult     *res,
+                    LauncherShortcut *shortcut)
+{
+  g_mount_unmount_finish (mount, res, NULL);
+  clutter_actor_set_opacity (CLUTTER_ACTOR (shortcut), 175);
+
+}
+
 static void
 on_eject_clicked (ClutterActor *shortcut, GVolume *volume)
 {
@@ -421,7 +653,8 @@ on_eject_clicked (ClutterActor *shortcut, GVolume *volume)
 
   if (g_volume_can_eject (volume))
   {
-    g_volume_eject (volume, 0, NULL, NULL, NULL);
+    g_volume_eject (volume, 0, NULL,
+                    (GAsyncReadyCallback)on_volume_ejected, shortcut);
   }
   else
   {
@@ -429,7 +662,8 @@ on_eject_clicked (ClutterActor *shortcut, GVolume *volume)
 
     if (!mount)
       return;
-    g_mount_unmount (mount, 0, NULL, NULL, NULL);
+    g_mount_unmount (mount, 0, NULL,
+                     (GAsyncReadyCallback)on_mount_unmounted, shortcut);
   }
 }
 
@@ -507,9 +741,17 @@ on_volume_removed (GVolume *volume, LauncherShortcut *shortcut)
 static void
 on_volume_changed (GVolume *volume, LauncherShortcut *shortcut)
 {
+  GMount *mount;
+
   g_return_if_fail (LAUNCHER_IS_SHORTCUT (shortcut));
   
   launcher_shortcut_set_can_eject (shortcut, can_eject (volume));
+
+  mount = g_volume_get_mount (volume);
+  if (G_IS_MOUNT (mount))
+    clutter_actor_set_opacity (CLUTTER_ACTOR (shortcut), 255);
+  else
+    clutter_actor_set_opacity (CLUTTER_ACTOR (shortcut), 175);
 }
 
 static void
@@ -525,9 +767,10 @@ on_volume_added (GVolumeMonitor *monitor,
   g_return_if_fail (LAUNCHER_IS_SIDEBAR (sidebar));
   priv = sidebar->priv;
 
-  g_debug ("Volume Added: %s", g_volume_get_name (volume));
-
-  mount = g_volume_get_mount (volume);
+  g_debug ("Volume Added: %s %s", 
+           g_volume_get_name (volume),
+           g_volume_get_identifier (volume, 
+                                    G_VOLUME_IDENTIFIER_KIND_HAL_UDI));
 
   icon_name = get_icon_name (volume);
   shortcut = launcher_shortcut_new (icon_name, 
@@ -537,7 +780,11 @@ on_volume_added (GVolumeMonitor *monitor,
   clutter_container_add_actor (CLUTTER_CONTAINER (priv->grid), shortcut);
   g_object_set_data (G_OBJECT (shortcut), "volume", volume);
   g_object_set_data (G_OBJECT (shortcut), "bar", sidebar);
-  clutter_actor_show (shortcut);
+  clutter_actor_show_all (shortcut);
+
+  mount = g_volume_get_mount (volume);
+  if (!G_IS_MOUNT (mount))
+    clutter_actor_set_opacity (shortcut, 175);
 
   g_signal_connect (shortcut, "clicked", 
                     G_CALLBACK (on_volume_clicked), volume);
@@ -587,6 +834,9 @@ launcher_sidebar_show_trash (LauncherSidebar *sidebar,
   g_return_if_fail (LAUNCHER_IS_SIDEBAR (sidebar));
   priv = sidebar->priv;
 
+  if (launcher_config_get_default ()->is_portrait)
+    show = FALSE;
+
   if (show)
   {
     if (CLUTTER_IS_ACTOR (priv->trash))
@@ -600,7 +850,7 @@ launcher_sidebar_show_trash (LauncherSidebar *sidebar,
                                                     FALSE);
   
     clutter_container_add_actor (CLUTTER_CONTAINER (priv->grid), shortcut);
-    clutter_actor_show (shortcut);
+    clutter_actor_show_all (shortcut);
     clutter_drag_dest_enable (CLUTTER_DRAG_DEST (priv->trash));
 
     g_signal_connect (priv->trash, "drop", 
@@ -613,7 +863,9 @@ launcher_sidebar_show_trash (LauncherSidebar *sidebar,
   else
   {
     priv->shortcuts = g_list_remove (priv->shortcuts, priv->trash);
-    clutter_actor_destroy (priv->trash);
+    if (CLUTTER_IS_ACTOR (priv->trash))
+      clutter_actor_destroy (priv->trash);
+    priv->trash = NULL;
   }
   
   ensure_layout (sidebar);
@@ -769,6 +1021,9 @@ launcher_sidebar_finalize (GObject *object)
 
   g_list_free (priv->shortcuts);
   g_list_free (priv->links);
+  g_list_free (priv->places);
+
+  g_slist_free (priv->excluded_volumes);
 
   G_OBJECT_CLASS (launcher_sidebar_parent_class)->finalize (object);
 }
@@ -780,6 +1035,15 @@ launcher_sidebar_class_init (LauncherSidebarClass *klass)
   
   obj_class->finalize = launcher_sidebar_finalize;
 
+	_sidebar_signals[SHORTCUTS_CHANGED] =
+		g_signal_new ("shortcuts-changed",
+			      G_OBJECT_CLASS_TYPE (obj_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (LauncherSidebarClass, shortcuts_changed),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID, 
+			      G_TYPE_NONE, 0);
+
   g_type_class_add_private (obj_class, sizeof (LauncherSidebarPrivate));
 }
 
@@ -795,9 +1059,9 @@ add_shortcut (LauncherSidebar *sidebar,
   shortcut = launcher_shortcut_new (icon_name, name, FALSE);
   clutter_container_add_actor (CLUTTER_CONTAINER (priv->grid), shortcut);
   clutter_actor_set_name (shortcut, command);
-  clutter_actor_show (shortcut);
+  clutter_actor_show_all (shortcut);
   priv->shortcuts = g_list_append (priv->shortcuts, shortcut);
-  g_signal_connect (shortcut, "clicked", G_CALLBACK (on_link_clicked), NULL);
+  g_signal_connect (shortcut, "clicked", G_CALLBACK (on_place_clicked), NULL);
   
   return shortcut;
 }
@@ -815,35 +1079,29 @@ load_volumes (LauncherSidebar *sidebar)
   g_return_if_fail (LAUNCHER_IS_SIDEBAR (sidebar));
   priv = sidebar->priv;
 
-  home = g_strdup_printf (NAUTILUS, g_get_home_dir ());
+  home = g_strdup_printf ("file://%s", g_get_home_dir ());
   shortcut = add_shortcut (sidebar, "gnome-fs-home", _("Home"), home);
   g_free (home);
-  shortcut = add_shortcut (sidebar, "gnome-fs-network", _("Network"), 
-                           "nautilus --no-desktop --browser Network:///");
-
+  shortcut = add_shortcut (sidebar, "gnome-fs-network", _("Network"),
+                           "Network:///");
+  
   volumes = g_volume_monitor_get_volumes (priv->monitor);
   for (vol = volumes; vol; vol = vol->next)
   {
     GVolume *volume;
-    gchar *icon_name;
-    
+     
     volume = vol->data;
-    icon_name = get_icon_name (volume);
-    shortcut = launcher_shortcut_new (icon_name, 
-                                      g_volume_get_name (volume),
-                                      can_eject (volume));
-    clutter_container_add_actor (CLUTTER_CONTAINER (priv->grid), shortcut);
-    clutter_actor_show (shortcut);
-    priv->shortcuts = g_list_append (priv->shortcuts, shortcut);
 
-    g_signal_connect (shortcut, "clicked", 
-                      G_CALLBACK (on_volume_clicked), volume);
-    g_signal_connect (shortcut, "eject", 
-                      G_CALLBACK (on_eject_clicked), volume);
-
-
-    g_free (icon_name);
+    if (volume_is_excluded (sidebar, volume))
+    {
+      g_object_unref (volume);
+      continue;
+    }
+    on_volume_added (priv->monitor, volume, sidebar);
+    
+    g_object_unref (volume);
   }
+  g_list_free (volumes);
 
   ensure_layout (sidebar);
 
@@ -854,7 +1112,7 @@ load_volumes (LauncherSidebar *sidebar)
   clutter_actor_set_position (shortcut, 0, 
                               cfg->win_height - (bar_height)-(PANEL_HEIGHT*2));
   clutter_actor_set_name (shortcut, "gnome-session-save --gui --kill");
-  clutter_actor_show (shortcut);
+  clutter_actor_show_all (shortcut);
   priv->links = g_list_append (priv->links, shortcut);
   g_signal_connect (shortcut, "clicked", G_CALLBACK (on_link_clicked), NULL);
 }
@@ -864,6 +1122,7 @@ launcher_sidebar_init (LauncherSidebar *sidebar)
 {
   LauncherSidebarPrivate *priv;
   LauncherConfig *cfg = launcher_config_get_default ();
+  GConfClient *client = gconf_client_get_default ();
   gchar *path;
   
   priv = sidebar->priv = LAUNCHER_SIDEBAR_GET_PRIVATE (sidebar);
@@ -887,6 +1146,11 @@ launcher_sidebar_init (LauncherSidebar *sidebar)
   priv->temp = clutter_effect_template_new (priv->timeline,
                                             clutter_sine_inc_func);
 
+  priv->excluded_volumes = gconf_client_get_list (client, 
+                                  "/apps/ume-launcher/volume_exclude_list",
+                                  GCONF_VALUE_STRING,
+                                  NULL);
+
   load_volumes (sidebar);
   load_places (sidebar);
 
@@ -907,6 +1171,8 @@ launcher_sidebar_init (LauncherSidebar *sidebar)
                     G_CALLBACK (launcher_sidebar_scroll_event), sidebar);
   g_signal_connect (priv->grid, "scroll-event",
                     G_CALLBACK (launcher_sidebar_scroll_event), sidebar);
+
+  g_object_unref (client);
 }
 
 ClutterActor *
